;; Symbolisches Differenzieren

;; Konstanten sind Zahlen
(defun konstante? (x) (number? x))


;; Eine Zahl kann eine ganze Zahl oder eine reele Zahl sein
(defun number? (x) 
        (or (equal (type-of x) 'FIXNUM) (equal (type-of x) 'SINGLE-FLOAT)))


;; Variablen sind Symbole
(defun variable? (x) (equal (type-of x) 'SYMBOL))

;; Zwei Variablen sind gleich, die die sie darstellenden Symbole in der 
;; Relation equal stehen
(defun gleiche-variable? (v1 v2) 
                      (and (variable? v1) (variable? v2) (equal v1 v2)))


;; Summen und Produkte werden als Listen definiert. Hierbei sind noch 
;; Vereinfachungen eingebaut.Ist z.B.ein Summand gleich Null, so ist das
;; Ergebnis gleich dem anderen Summanden.
(defun konstr-summe (a1 a2) 
  (cond ((and (number? a1) (number? a2)) (+ a1 a2))
        ((number? a1) (if (= a1 0) a2
                                   (list '+ a1 a2)))
        ((number? a2) (if (= a2 0) a1
                                   (list '+ a1 a2)))
        (t (list '+ a1 a2))))
  

(defun konstr-produkt (a1 a2) 
    (cond ((and (number? a1) (number? a2)) (* a1 a2))
          ((number? a1) 
            (cond ((= a1 0) 0)
                  ((= a1 1) a2)
                  (t (list '* a1 a2))))
          ((number? a2) 
            (cond ((= a2 0) 0)
                  ((= a2 1) a1)
                  (t (list '* a1 a2))))
          (t (list '* a1 a2))))
      
  

;; Eine Summe ist eine Liste, deren erstes Element das Symbol '+' ist.
(defun summe? (x) (and (not (atom x)) (equal (car x) '+))) 


;; Der erste Summand ist das zweite Element der Summenliste.
(defun summand1 (s) (cadr s))

;; Der zweite Summand ist das dritte Element der Summenliste.
(defun summand2 (s) (caddr s))


;; Ein Produkt ist eine Liste, deren erstes Element das Symbol '*' ist.
(defun produkt? (x) (and (not (atom x)) (equal (car x) '*))) 


;; Der Multiplikator ist das zweite Element der Produktliste.
(defun multiplikator (p) (cadr p))


;; Der Multiplikand ist das dritte Element der Produktliste.
(defun multiplikand (p) (caddr p))


;; Mit Hilfe der obigen Operatoren kann nun die Funktion Ableitung definiert
;; werden. Sie gibt die Regeln fuer das Differenzieren von Ausdruecken wieder,
;; die nur aus Summen, Produkten, Konstanten und Variablen aufgebaut sind.
(defun ableitung (ausdr var)
  (cond ((konstante? ausdr) 0)
        ((variable? ausdr)
         (if (gleiche-variable? ausdr var) 1 0))
        ((summe? ausdr)
         (konstr-summe (ableitung (summand1 ausdr) var)
                       (ableitung (summand2 ausdr) var)))
        ((produkt? ausdr)
         (konstr-summe
           (konstr-produkt
             (multiplikator ausdr)
             (ableitung (multiplikand ausdr) var))
           (konstr-produkt
             (ableitung (multiplikator ausdr) var) 
             (multiplikand ausdr))))))
           