MODULE CD;

(* Implementierung des abstrakten Datentyps CD.
   Autor           : Thomas von der Maen, RWTH Aachen
   Umgebung        : PM-3 Windows 2000
   Erstellt        : 19.12.00  Letzte Aenderung: 02.01.01 
*)

IMPORT SIO;     
IMPORT Track;    (* Importiere den ADT Track *)


(* Die Trackliste wird als lineare Liste realisiert *)
TYPE Tracklist = REF RECORD
                   track: Track.T;
                   next: Tracklist;
                 END;

(* Eine CD besteht aus einem Interpreten, einem Titel und einer Liste von
Musikstcken *)
     REVEAL T = BRANDED REF RECORD
                  interpret: TEXT;
                  title: TEXT;
                  tracklist: Tracklist;
                END;

PROCEDURE GetInterpret(cd: T): TEXT =
(* Liefert den Interpreten einer CD *)
BEGIN
  RETURN (cd^.interpret);
END GetInterpret;


PROCEDURE SetInterpret(VAR cd: T; interpret: TEXT) =
(* Setzt den Interpreten der angegebenen CD *)
BEGIN
  cd^.interpret := interpret;
END SetInterpret;


PROCEDURE GetTitle(cd: T): TEXT =
(* Liefert den Titel der bergebenen CD *)
BEGIN
  RETURN (cd^.title);
END GetTitle;


PROCEDURE SetTitle(VAR cd: T; title: TEXT) =
(* Setzt den Titel der bergebenen CD *)
BEGIN
  cd^.title := title;
END SetTitle;


PROCEDURE GetTotalTime(cd: T): CARDINAL =
(* Liefert die Gesamtspielzeit der CD in Sekunden *)
VAR totaltime: CARDINAL;
    tlist : Tracklist;
    t : Track.T;
BEGIN
  tlist := cd^.tracklist;
  totaltime := 0;
  (* Summiere die Lngen der einzelnen Tracks der CD *)
  WHILE (tlist # NIL) DO
    t := tlist^.track;
    totaltime := totaltime + Track.GetLength(t);
    tlist := tlist^.next;
  END;
  RETURN totaltime;
END GetTotalTime;

PROCEDURE AddTrack(VAR cd: T; track: Track.T) =
(* Fgt einen Track der bergebenen CD hinzu. Der neue Track wird
an das Ende der Trackliste angehngt. *)
VAR newtrack, actual, prev: Tracklist;
BEGIN
  newtrack := NEW(Tracklist);
  newtrack^.track := track;
  newtrack^.next := NIL;

  (* Falls Trackliste leer *)
  IF (cd^.tracklist = NIL) THEN
    cd^.tracklist := newtrack;
  ELSE
    (* Suche Ende der Trackliste *)
    actual := cd^.tracklist;
    prev := cd^.tracklist;

    WHILE (actual # NIL) DO
      prev := actual;
      actual := actual^.next;
    END;

    prev^.next := newtrack;
  END;
END AddTrack;
  

PROCEDURE RemoveTrack(VAR cd: T) =
(* Lscht den letzten Track der bergebenen CD *)
VAR actual, prev: Tracklist;
BEGIN
  IF (cd^.tracklist = NIL) THEN
    SIO.PutLine("Trackliste ist leer!");
  ELSIF (cd^.tracklist.next = NIL) THEN
    cd^.tracklist := NIL;
  ELSE
    actual := cd^.tracklist;
    prev := cd^.tracklist;
    WHILE (actual^.next # NIL) DO
      prev := actual;
      actual := actual^.next;
    END;
    prev^.next := NIL;
  END;
END RemoveTrack;


PROCEDURE GetTrackCount(cd: T): CARDINAL =
(* Liefert die Anzahl der Tracks der bergebenen CD *)
VAR count: CARDINAL;
    tl: Tracklist;
BEGIN
  count := 0;
  tl := cd^.tracklist;
  
  WHILE (tl # NIL) DO
    count := count + 1;
    tl := tl^.next;
  END;
  RETURN count;
END GetTrackCount;


PROCEDURE PrintTracklist(cd: T) =
(* Zeigt die Trackliste der CD mit Namen und Lnge der Tracks 
auf dem Bildschirm an *)
VAR tl: Tracklist;
    count : CARDINAL;
BEGIN
  tl := cd^.tracklist;
  count := 1;
  WHILE (tl # NIL) DO
    SIO.PutInt(count); SIO.PutText(". ");
    SIO.PutText(Track.GetName(tl^.track) & " Dauer: ");
    SIO.PutInt(Track.GetLength(tl^.track)); SIO.Nl();
    tl := tl^.next;
    count := count + 1;
  END;
END PrintTracklist;


PROCEDURE CreateCD(): T =
(* Erstellt eine neue CD und initalisiert diese *)
VAR newcd: T;
BEGIN
  newcd := NEW(T);
  newcd^.interpret := "";   (* Initialisiere Interpreten *)
  newcd^.title := "";       (* Initialisiere Titel *)
  newcd^.tracklist := NIL;  (* Initialisiere Trackliste *)
  RETURN newcd;
END CreateCD;

BEGIN
END CD.