MODULE Sequentiell;

(* Modul zur Klasse Sequentiell
   Autor           : Antje Nowack, Thomas van der Massen
   Umgebung  : PM-3 Windows 95
   Erstellt        : 19.12.00  Letzte Aenderung: 22.01.01 
*)

IMPORT Track;
IMPORT Tontraeger;
IMPORT SIO;
IMPORT Text;
IMPORT Fmt;

REVEAL
   T = Public BRANDED OBJECT
                      actual: Tontraeger.Tracklist := NIL;
                   OVERRIDES                                  
                    Play := PlayerSeq;
                    (* Abspielen des sequentiellen Tontraegers an der aktuellen Stelle *)
                    Spulen := SpulenSeq;
                    (* Spulen des Tontraegers *)
                    PrintInfo := PrintSeq;
                    (* Ausgabe aller Informationen ueber den Tontraeger als Text. Dies ist eine Redefinition der 
                       entsprechenden Methode des Typen Tontraeger. *)
                    AddTrack := AddTrackSeq;
                    (* Fgt einen Track des bergebenen Tontraegers hinzu. Dies ist eine Redefinition der 
                       entsprechenden Methode des Typen Tontraeger. *)
                    Ende := EndeSeq;
                    (* Ueberprueft, ob der Tontraeger an seinem Ende angelangt ist. *)
                   END;


PROCEDURE SpulenSeq(self: T) =
(* Spulen des Tontraegers *)
VAR act : Tontraeger.Tracklist;
BEGIN
  (* Die Trackliste wird von Anfang an so lange durchlaufen, bis der Vorgraenger des aktuellen Tracks gefunden 
     ist. Der aktuelle Trakc wird durch seinen Vorgaenger ersetzt. *)
   act := self.GetTracklist();
    (* Wenn der aktuelle Track nicht der erste Track ist und somit der Track keinen Vorgraenger hat, der Tontraeger 
     also nicht bereits am Anfang steht ... *)
  IF NOT self.Ende() THEN  
    IF NOT ( Text.Equal( ((self.actual)^.track).GetName(), ((self.GetTracklist())^.track).GetName()) 
                    AND ( ((self.actual)^.track).GetLength () =  ((self.GetTracklist())^.track).GetLength()))
    THEN
       (* ... dann suche den Vorgaenger des aktuellen Tracks ... *)
       WHILE  NOT (Text.Equal( ((act^.next)^.track).GetName(), ((self.actual)^.track).GetName()) 
                              AND ( ((act^.next)^.track).GetLength () =  ((self.actual)^.track).GetLength())) DO
           act := act^.next;
        END;
        self.actual :=act;
        (* ... sonst ist Spulen nicht moeglich. *)

        ELSE SIO.PutLine("Spulen nicht moeglich! Der Tontraeger ist bereits am Anfang.");
    END;
  ELSIF self.GetTracklist() = NIL THEN SIO.PutLine("Die Trackliste ist leer!");
  ELSE 
    WHILE  NOT act.next = NIL DO
          act := act^.next;
        END;
        self.actual :=act;
  END;
END SpulenSeq;

PROCEDURE PlayerSeq(self : T) : Track.T =
(* Abspielen des sequentiellen Tontraegers an der aktuellen Stelle *)
VAR play: Tontraeger.Tracklist;
BEGIN
  (* Liefere den aktuellen Track zurueck ... *)
  play := self.actual;
  (* ... und ersetze den aktuellen Track durch seinen Vorgaenger ... *)
  self.actual := (self.actual)^.next;
  RETURN play.track;
END PlayerSeq;

PROCEDURE EndeSeq(self:T) : BOOLEAN =
(* Ueberprueft, ob der Tontraeger an seinem Ende angelangt ist. *)
BEGIN
  RETURN self.actual = NIL;
END EndeSeq;

PROCEDURE PrintSeq(self : T) : TEXT =
(* Ausgabe aller Informationen ueber den Tontraeger als Text. Dies ist eine Redefinition der 
   entsprechenden Methode des Typen Tontraeger. *)

BEGIN
  (* Liefert zusaetzlich zu den Informationen, die die Methode des Supertypen Tontraeger liefert, 
     die Zugriffsart. *)
  IF NOT self.actual = NIL THEN
    RETURN Tontraeger.T.PrintInfo(self) &   " Zugriff : Sequentiell  " 
                      & "\n" & "       " & " aktueller Track: " & (((self.actual)^.track).GetName()) 
                      & "   Laenge: " & Fmt.Int ((((self.actual)^.track).GetLength ())) & " Sekunden" &"\n" & "    ";
  ELSE 
    RETURN Tontraeger.T.PrintInfo(self) &   " Zugriff : Sequentiell  " 
                      & "\n" & "       " & "Der Tontaeger befindet sich am Ende." &"\n" & "    ";
  END;
END PrintSeq;

PROCEDURE AddTrackSeq(self: T; track: Track.T) =
(* Fgt einen Track des bergebenen Tontraegers hinzu. Dies realisiert die Redefinition der entsprechenden 
   Methode des Supertypen Tontraeger. Ausser der Aktualisierung der Trackliste ist hier auch die Aktualisierung 
   der Liste notwendig, die den aktuellen Track angibt. Deren Aktualisierung wird genauso realisiert wie die
   der Trackliste. *)

VAR newtrack, act, prev : Tontraeger.Tracklist;
VAR add : BOOLEAN := TRUE;
BEGIN 
  Tontraeger.T.AddTrack(self, track);
  newtrack := NEW(Tontraeger.Tracklist);
  newtrack^.track := track;
  newtrack^.next := NIL;
  
  (* Falls actual-Liste leer - dies ist nur der Fall, wenn zuvor die Trackliste leer war *)

  IF (self.actual = NIL) THEN
    self.actual := newtrack;
  ELSE
    (* Suche Ende der Trackliste *)
    act := self.actual;
    prev := self.actual;
    WHILE (act # NIL) DO
      IF Text.Equal(track.GetName() , (act^.track).GetName()) AND track.GetLength() = (act^.track).GetLength()
      THEN
          act := NIL;
          add := FALSE;
      ELSE
        prev := act;
        act := act^.next;
      END;
    END;
    IF add THEN prev^.next := newtrack; END;
  END;
END AddTrackSeq;

BEGIN
END Sequentiell.
