MODULE Tontraeger;

(* Modul zur Klasse Tontraeger
   Autor           : Antje Nowack, Thomas van der Massen
   Umgebung : PM-3 Windows 95
   Erstellt        : 19.12.00  Letzte Aenderung: 22.01.01 
*)

IMPORT Track;
IMPORT SIO;
IMPORT Text;
IMPORT Fmt;

REVEAL
      T = Public BRANDED OBJECT
                  interpret: TEXT;
                  title: TEXT;
                  tracklist: Tracklist := NIL;
            OVERRIDES
                    GetInterpret := GetInterpretTt;
                    (* Liefert den Interpreten des Tontraegers *)

                    SetInterpret := SetInterpretTt;
                    (* Setzt den Interpreten des Tontraegers *)

                    GetTitle := GetTitleTt;
                    (* Liefert den Titel de Tontraegers *)

                    SetTitle := SetTitleTt;
                    (* Setzt den Titel des bergebenen Tontraegers *)

                    AddTrack := AddTrackTt;
                    (* Fgt einen Track dem Tontraegers hinzu*)

                    GetTrackCount := GetTrackCountTt;
                    (* Liefert die Anzahl der Tracks des Tontraegers *)

                    PrintTracklist := PrintTracklistTt;
                    (* Zeigt die Trackliste des Tontraegers auf dem Bildschirm an *)

                    GetTracklist := GetTracklistTt;
                    (* Liefert die Liste der Tracks des Tontraegers. *)

                    PrintInfo := PrintTt;
                    (* Liefert einen Text, der alle Informationen - inklusive Trackliste - ueber 
                       den Tontraeger enthaelt. *)

                    TracklistAsText := TracklistAsTextTt;
                    (* Liefert die Trackliste des Tontraegers mit Namen und Lnge der Tracks als Text. *)
            
           END;


PROCEDURE GetInterpretTt(self: T): TEXT =
(* Liefert den Interpreten des Tontraegers *)
BEGIN
  RETURN (self.interpret);
END GetInterpretTt;


PROCEDURE SetInterpretTt(self: T; interpret: TEXT) =
(* Setzt den Interpreten des Tontraegers *)
BEGIN
  self.interpret := interpret;
END SetInterpretTt;


PROCEDURE GetTitleTt(self: T): TEXT =
(* Liefert den Titel des Tontraegers *)
BEGIN
  RETURN (self.title);
END GetTitleTt;


PROCEDURE SetTitleTt(self: T; title: TEXT) =
(* Setzt den Titel des Tontraegers *)
BEGIN
  self.title := title;
END SetTitleTt;


PROCEDURE AddTrackTt(self: T; track: Track.T) =
(* Fgt einen Track dem Tontraeger hinzu. Der neue Track wird
an das Ende der Trackliste angehngt. *)
VAR newtrack, actual, prev: Tracklist;
VAR add : BOOLEAN := TRUE;
BEGIN
  newtrack := NEW(Tracklist);
  newtrack^.track := track;
  newtrack^.next := NIL;

  (* Falls Trackliste leer *)
  IF (self.tracklist = NIL) THEN
    self.tracklist := newtrack;
  ELSE
    (* Suche Ende der Trackliste *)
    actual := self.tracklist;
    prev := self.tracklist;
    WHILE (actual # NIL) DO
      (* Falls der einzufuegende Track bereits an der betrachteten Stelle steht *)
      IF Text.Equal(track.GetName() , (actual^.track).GetName()) AND track.GetLength() = (actual^.track).GetLength()
      THEN
        (* wird der Tontraeger nicht veraendert. *)
        actual := NIL;
        add := FALSE;
        SIO.PutLine("Der angegebene Track ist bereits enthalten.");
      ELSE
        (* sonst Aenderung gemaess obiger Beschreibung *)
        prev := actual;
        actual := actual^.next;
      END;
    END;
    IF add THEN prev^.next := newtrack; END;
  END;
END AddTrackTt;
  

PROCEDURE GetTrackCountTt(self: T): CARDINAL =
(* Liefert die Anzahl der Tracks des Tontraegers *)
VAR count: CARDINAL;
    tl: Tracklist;
BEGIN
  count := 0;
  tl := self.tracklist;
  
  WHILE (tl # NIL) DO
    count := count + 1;
    tl := tl^.next;
  END;
  RETURN count;
END GetTrackCountTt;


PROCEDURE PrintTracklistTt(self: T) =
(* Zeigt die Trackliste des Tontraegers mit Namen und Lnge der Tracks 
auf dem Bildschirm an. Dies wird realisiert durch einen rekursiven durchlauf durch die Liste. *)
VAR tl: Tracklist;
    count : CARDINAL;
BEGIN
  tl := self.tracklist;
  count := 1;
  WHILE (tl # NIL) DO
    SIO.PutInt(count); SIO.PutText(". ");
    SIO.PutText(Track.T.GetName(tl^.track) & " Dauer: ");
    SIO.PutInt(Track.T.GetLength(tl^.track)); SIO.Nl();
    tl := tl^.next;
    count := count + 1;
  END;
END PrintTracklistTt;

PROCEDURE GetTracklistTt(self : T) : Tracklist =
(* Liefert die Liste der Tracks des Tontraegers. *)
BEGIN
  RETURN self.tracklist;
END GetTracklistTt;

PROCEDURE TracklistAsTextTt(self : T) : TEXT = 
(* Liefert die Trackliste des Tontraegers mit Namen und Lnge der Tracks 
   als Text. Dies wird realisiert durch einen rekursiven durchlauf durch die Liste. *)

VAR tl: Tracklist;
    count : CARDINAL;
    result : TEXT;
BEGIN
  tl := self.tracklist;
  count := 1;
  result := "        ";
  WHILE (tl # NIL) DO
    result := result & Fmt.Int(count) & ". " & (tl^.track).GetName() & " Dauer: " & Fmt.Int(Track.T.GetLength(tl^.track))  & " Sekunden" & "\n" & "        "; 
    tl := tl^.next;
    count := count + 1;
  END;
  RETURN result & "\n" & "    ";
END TracklistAsTextTt;


PROCEDURE PrintTt(self : T) : TEXT =
(* Liefert einen Text, der alle Informationen - inklusive Trackliste - ueber den Tontraeger enthaelt. *)
BEGIN
   RETURN "Interpret: " & self.interpret & "   Titel: " & self. title  & "\n" & self.TracklistAsText() ;
END PrintTt;  

BEGIN
END Tontraeger.
