MODULE Mengenrelationen EXPORTS Main;

(* Dieses Programm realisiert Mengenrelationen.
   Autor           : Thomas von der Maen, RWTH Aachen
   Umgebung        : PM-3, Windows 2000
   Erstellt        : 29.11.00  Letzte Aenderung: 30.11.00
*)


IMPORT SIO;

(* Deklariere Datentypen *)
TYPE Zeichen = ['a' .. 'z'];
     Buchstaben = SET OF Zeichen;
   
VAR menge1, menge2 := Buchstaben{};
    alle := Buchstaben{'a' .. 'z'};
    z : CHAR;
    ergebnis : BOOLEAN;


PROCEDURE Ausgabe(m: Buchstaben)=
(* Diese Prozedure gibt alle Elemente einer Menge aus *)
BEGIN
  FOR i:=FIRST(Zeichen) TO LAST(Zeichen) DO
    IF i IN m THEN
      SIO.PutChar(i);
      SIO.PutChar(' ');
    END;
  END;
END Ausgabe;


PROCEDURE Einlesen(VAR m: Buchstaben)=
(* Diese Prozedur liest solange Zeichen von der Tastatur ein und legt diese in einer Menge ab,
bis '!' eingegeben wird. *)
BEGIN
  REPEAT
    z := SIO.GetChar();
    IF z IN alle THEN
      m := m + Buchstaben{z};
    END;
  UNTIL z = '!';
END Einlesen;


PROCEDURE DruckeMenu()=
(* Diese Prozedur zeigt das Benutzermen auf dem Bildschirm an. *)
BEGIN
  SIO.PutText("+----------------------+"); SIO.Nl();
  SIO.PutText("| 1 - Gleichheit       |"); SIO.Nl();
  SIO.PutText("| 2 - Ungleichheit     |"); SIO.Nl();
  SIO.PutText("| 3 - Teilmenge        |"); SIO.Nl();
  SIO.PutText("| 4 - Echte Teilmenge  |"); SIO.Nl();
  SIO.PutText("| 5 - Obermenge        |"); SIO.Nl();
  SIO.PutText("| 6 - Echte Obermenge  |"); SIO.Nl();
  SIO.PutText("| 7 - Beenden          |"); SIO.Nl();
  SIO.PutText("+----------------------+"); SIO.Nl();
  SIO.PutText("Bitte treffen Sie eine Auswahl: ");
END DruckeMenu;

PROCEDURE Gleich(m1, m2: Buchstaben): BOOLEAN =
(* Prft, ob die Mengen m1 und m2 Mengen gleich sind *)
BEGIN
  RETURN (Teilmenge(m1, m2) AND Teilmenge(m2, m1));
END Gleich;

PROCEDURE Ungleich(m1, m2: Buchstaben): BOOLEAN =
(* Prft ob die Mengen m1 und m2 ungleich sind *)
BEGIN
  RETURN NOT(Gleich(m1, m2));
END Ungleich;

PROCEDURE Teilmenge(m1, m2: Buchstaben): BOOLEAN =
(* Prft, ob die Menge m1 eine Teilmenge der Menge m2 ist *)
VAR istGleich: BOOLEAN;
BEGIN
  istGleich := TRUE;
  FOR i:=FIRST(Zeichen) TO LAST(Zeichen) DO
    (* Falls das Element i in m1 ist, dann muss es auch in m2 vorhanden sein *)
    IF i IN m1 THEN
      istGleich := istGleich AND (i IN m2);
    END;
  END;
  RETURN istGleich;
END Teilmenge;

PROCEDURE EchteTeilmenge(m1, m2: Buchstaben): BOOLEAN =
(* Prft, ob die Menge m1 eine echte Teilmenge der Menge m2 ist *)
BEGIN
  RETURN (Teilmenge(m1, m2) AND Ungleich(m1, m2));
END EchteTeilmenge;

PROCEDURE Obermenge(m1, m2: Buchstaben): BOOLEAN =
(* Prft, ob die Menge m1 eine Obermenge der Menge m2 ist *)
BEGIN
  RETURN Teilmenge(m2, m1);
END Obermenge;

PROCEDURE EchteObermenge(m1, m2: Buchstaben): BOOLEAN =
(* Prft, ob die Menge m1 eine echte Obermenge der Menge m2 ist *)
BEGIN
  RETURN EchteTeilmenge(m2, m1);
END EchteObermenge;

BEGIN
  SIO.PutText("Geben Sie bitte die Elemente der 1. Menge ein (Abschliessen mit '!'):");
  SIO.Nl();
  Einlesen(menge1);
  SIO.PutText("Geben Sie bitte die Elemente der 2. Menge ein (Abschliessen mit '!'):");
  SIO.Nl();
  Einlesen(menge2);

  SIO.PutText("Die Elemente der 1. Menge lauten: ");
  Ausgabe(menge1);
  SIO.Nl();
  SIO.PutText("Die Elemente der 2. Menge lauten: ");
  Ausgabe(menge2);
  SIO.Nl();

  ergebnis := TRUE;
  REPEAT
    DruckeMenu();
    z := SIO.GetChar();
    z := SIO.GetChar();
    z := SIO.GetChar();
    CASE z OF
      '1' => ergebnis := Gleich(menge1, menge2); |
      '2' => ergebnis := Ungleich(menge1, menge2); |
      '3' => ergebnis := Teilmenge(menge1, menge2); |
      '4' => ergebnis := EchteTeilmenge(menge1, menge2); |
      '5' => ergebnis := Obermenge(menge1, menge2); |
      '6' => ergebnis := EchteObermenge(menge1, menge2); |
      '7' => SIO.PutText("Ende");
    ELSE
        SIO.PutText("Keine gltige Auswahl !!!");
        SIO.Nl();
    END; 
    IF z # '7' THEN
      SIO.PutText("Ergebnis: "); SIO.PutBool(ergebnis); SIO.Nl();
    END;
  UNTIL z = '7';

END Mengenrelationen.