MODULE Zykl_Liste EXPORTS Main;

(* Autor: Antje Nowack
   Umgebung: PM3, Windows95
   Erstellt: 16. 12. 2000
   Letzte Aenderung: 18. 12. 2000
*)

IMPORT SIO;

(*-------------------------Aufgabenteil a-------------------------*)

(* Definition der Datenstruktur. Der Datensatz besteht aus einer Zahl
   vom Typ Integer. Jedes Element besitzt einen Nachfolger und einen
   Vorgaenger. *)

TYPE Link = REF Element;
     Element = RECORD
                 nr : INTEGER;
                 vor, nach : Link;
               END;

(* Die benutzten globalen Variablen werden definiert. kette ist der Anker
   zur aktuellen Liste.
   wahl enthaelt die aktuelle Wahl im Hauptprogramm.
   neu soll beim Einfuegen die einzufuegende Zahl enthalten.
   dummy dient zum Abfangen der Probleme mit SIO.GetInt().
   a,b steuern die Zerstoerung/Beschaedigung der Verkettung einer Liste. *)

VAR kette : Link;
    wahl, neu : INTEGER;
    dummy : TEXT;
    a,b : CARDINAL;



PROCEDURE SucheLetzten(kette : Link) : Link =
(* Diese Funktion liefert zu einer Liste den Vorgaenger des Ankers. *)

(* aktuell enthaelt das aktuell betrachtete Listenelement. *)
VAR aktuell : Link;
  BEGIN
    (* Der Fall der leeren Liste wird nicht betrachtet, da diese
       Funktion nicht mit der leeren Liste aufgerufen wird. *)
    (* Wenn die Liste aus mehr aus einem Element besteht ... *)
    IF kette.nach # kette THEN
      aktuell := kette^.nach;
      (* Die Schleife wird so lange ausgefuehrt, bis der Vorgaenger des
         Ankers erreicht ist ... *)
      WHILE aktuell^.nach # kette DO
      (* jeweils das naechste Listenelement wird betrachtet ... *)
        aktuell := aktuell^.nach;
      END;
      (* ... und der Vorgaenger des Ankers zurueckgegeben. *)
      RETURN aktuell;
    ELSE
     RETURN kette;
  END;
END SucheLetzten;


PROCEDURE Einfuegen(VAR kette : Link;  neu : INTEGER)=
(* Diese Prozedur fuegt in eine Liste eine Zahl ein. Hierbei wird
   das einzufuegende Element Vorgaenger des Ankers (und Nachfolger des
   urspruenglichen Vorgaengers des Ankers). Der urspruengliche Anker bleibt
   der Anker. *)

VAR elem : Link;
BEGIN
  elem := NEW(Link);
  elem^.nr := neu;
  IF kette = NIL THEN
    elem^.nach := elem;
    elem^.vor := elem;
    kette := elem;
  ELSE
    elem^.nach := kette;
    elem^.vor := SucheLetzten(kette);
    SucheLetzten(kette)^.nach := elem;
    kette^.vor := elem;
  END;
END Einfuegen;


PROCEDURE AusgebenVorwaerts (kette : Link) =
(* Diese Prozedur gibt eine Liste auf dem Bildschirm aus, indem 
   zuerst der Inhalt des Ankers ausgegeben und dann jeweils der des
   Nachfolgers des aktuellen Elementes, bis der Vorgaenger des Ankers
   erreicht ist. *)

VAR aktuell : Link;
BEGIN
  IF kette # NIL THEN 
    SIO.PutInt(kette^.nr);
    SIO.Nl();
    dummy := SIO.GetLine(); 
    IF kette.nach # kette THEN
      aktuell := kette^.nach;
      WHILE aktuell # kette DO
        SIO.PutInt(aktuell^.nr);
        SIO.Nl();
        dummy := SIO.GetLine();
        aktuell := aktuell^.nach;
      END;
    END;
  END;  
END AusgebenVorwaerts;


PROCEDURE AusgebenRueckwaerts (kette : Link) =
(* Diese Prozedur gibt eine Liste auf dem Bildschirm aus, indem 
   zuerst der Inhalt des Ankers ausgegeben und dann jeweils der des
   Vorgaengers des aktuellen Elementes bis der Nachfolger des Ankers
   erreicht ist. *)

VAR aktuell : Link;
BEGIN
  IF kette # NIL THEN 
    SIO.PutInt(kette^.nr);
    SIO.Nl();
    dummy := SIO.GetLine(); 
    IF kette.vor # kette THEN
      aktuell := kette^.vor;
      WHILE aktuell # kette DO
        SIO.PutInt(aktuell^.nr);
        SIO.Nl();
        dummy := SIO.GetLine(); 
        aktuell := aktuell^.vor;
      END;
    END;
  END;  
END AusgebenRueckwaerts;


PROCEDURE Zerstoeren(VAR kette : Link; a, b : CARDINAL) =
(* Mit Hilfe dieser Prozedur kann die Verkettung einer Liste
   beschaedigt werden. Uebergeben wird die Kette (mit Hilfe eines
   Referenzparameters) und zwei natuerliche Zahlen a und b.
   Der b-te Nachfolger des Ankers wird zum Nachfolgers des a-ten Nachfolgers
   des Ankers. *)

VAR element_a, element_b : Link;
BEGIN
  element_a := kette;
  element_b := kette;
  FOR i := 0 TO a DO
    element_a := element_a^.nach;
  END;
  FOR i := 0 TO b DO
    element_b := element_b^.nach;
  END;
  element_a^.nach := element_b;
END Zerstoeren;

(*-------------------------Aufgabenteil b-------------------------*)


PROCEDURE Test(kette: Link): BOOLEAN=
(* Diese Funktion liefert zu einer doppelt verketteten zyklischen
   Liste einen booleschen Wert. Dieser ist TRUE gdw. die Verkettung nicht
   beschaedigt ist. *)

VAR vorElement, nachElement: Link;
    verkettungOK           : BOOLEAN;
BEGIN
  (* Zunaechst den Sonderfall der leeren Liste abfangen. *)
  IF kette = NIL THEN
    RETURN TRUE;
  ELSE
    (* Der Test laesst sich auf eine lokale Bedingung reduzieren.
       Wenn NIL auftritt als Nachfolger oder wenn der Vorgaenger des
       Nachfolgers eines Elementes nicht das Element selber ist, so ist die
       Verkettung beschaedigt. Andere Faelle koennen nicht auftreten. *)
    nachElement := kette;
    vorElement  := kette^.vor;  (* Nicht leere Kette! *)
    verkettungOK := TRUE;
    (* Ueberpruefung der obigen lokalen Bedingung in einer Schleife ... *)
    REPEAT
      IF (vorElement = NIL) OR 
         (nachElement # vorElement^.nach) THEN
         verkettungOK := FALSE;
      ELSE
        nachElement := vorElement;
        vorElement := vorElement^.vor;
      END;
    (* ... bis eine Stelle gefunden wurde, die die Beschaedigung der
       Verkettung belegt oder die gesamte Liste durchlaufen wurde (ohne
       eine solche Stelle zu finden). *)
    UNTIL (vorElement = kette^.vor) OR NOT verkettungOK;
    RETURN verkettungOK;
  END;
END Test;

(*-------------------------Hauptprogramm-------------------------*)

(* Steuerung ueber ein Menue *)

BEGIN
  kette := NIL;
  WHILE wahl # 6 DO
    SIO.Nl();
    SIO.PutLine("************************** Menue **************************");
    SIO.Nl();
    SIO.PutLine("Waehlen Sie einen Menuepunkt:");
    SIO.Nl();
    SIO.PutLine("Zahl zur Liste hinzufuegen (1)");
    SIO.Nl();
    SIO.PutLine("Liste vorwaerts ausgeben (2)");
    SIO.Nl();
    SIO.PutLine("Liste rueckwaerts ausgeben (3)");
    SIO.Nl();
    SIO.PutLine("Zerstoeren der Verkettung (4)");
    SIO.Nl();
    SIO.PutLine("Ueberpruefung der Verkettung (5)");
    SIO.Nl();
    SIO.PutLine("Programm beenden (6)");
    SIO.Nl(); 
    SIO.PutLine("************************** Ende Menue **************************");
    SIO.Nl();
    SIO.PutText("Wahl: ");
    wahl := SIO.GetInt();
    dummy := SIO.GetLine();

    IF wahl = 1 THEN
      SIO.PutText("Bitte geben Sie die einzufuegende Zahl ein: ");
      neu := SIO.GetInt();
      dummy := SIO.GetLine();
      Einfuegen(kette, neu);

    ELSIF wahl = 2 THEN
      SIO.PutLine("Die Liste sieht vorwaerts wie folgt aus: ");
      AusgebenVorwaerts(kette);
      SIO.PutLine("Hier schliesst sich der Zykel. ");

    ELSIF wahl = 3 THEN
      SIO.PutLine("Die Liste sieht rueckwaerts wie folgt aus: ");
      AusgebenRueckwaerts(kette);
      SIO.PutLine("Hier schliesst sich der Zykel. ");

    ELSIF wahl = 4 THEN
      SIO.PutText("Bitte geben Sie die erste Position an: ");
      a := SIO.GetInt();
      dummy := SIO.GetLine();
      SIO.PutText("Bitte geben Sie die zweite Position an: ");
      b := SIO.GetInt();
      dummy := SIO.GetLine();
      Zerstoeren(kette, a, b);

   ELSIF wahl = 5 THEN
      IF Test(kette) THEN
        SIO.PutLine("Die Verkettung ist NICHT BESCHAEDIGT.");
      ELSE
        SIO.PutLine("Die Verkettung ist BESCHAEDIGT.");
      END;

    ELSIF wahl # 6 THEN
      SIO.PutLine("Keine korrekte Eingabe. Bitte wiederholen.");
    END;
  END;
  SIO.PutLine("Programmende!");
END Zykl_Liste.
