MODULE TerminDatei EXPORTS Main;

(* 
   Dieses Programm realisiert eine simple Terminverwaltung, die
   zuvor eingegebene Termine sortiert ausgibt. Die Termine koennen
   ausserdem in einer Datei gespeichert und von dort wieder gelesen
   werden.

   Autor           : Moritz Schnizler, RWTH Aachen
   Umgebung        : PM 3, Windows NT 4.0
   Erstellt        : 23.11.00
   Letzte Aenderung: 19.01.01
*)

IMPORT SIO;    (* Importiere notwendige Ein-/Ausgabeoperationen *)
IMPORT IO;     (* Importiere notwendige Operationen um Dateien zu oeffnen*)
IMPORT Rd, Wr; (* Importiere Operationen um Dateien zu lesen bzw. zu schreiben *)

CONST MAXTERMIN = 10;         (* Maximale Anzahl der zu verwaltenden Termine *)
      KALENDER  = "Kalender"; (* Name der Ausgabedatei *)

TYPE (* Unterbereichtstypen und Typ Uhrzeit fuer die Zeitangabe *)

     Stunden        = [0..23];

     Minuten        = [0..59];

     Uhrzeit        = RECORD
                        stunde: Stunden;
                        minute: Minuten;
                      END; 


     (* Typ Termin fuer einen einzelnen Termin mit Uhrzeit und Beschreibung *)

     Termin         = RECORD
                        beschreibung: TEXT;
                        zeit        : Uhrzeit;
                      END;
    
     
     (* Typ Terminkalender fuer maximal MAXTERMIN Termine *)

     Terminkalender = ARRAY [1..MAXTERMIN] OF Termin;


PROCEDURE TerminEingeben(): Termin =
(* Eingeben eines einzelnen Termins *)
VAR ergTermin: Termin;

BEGIN
  SIO.PutText("Beschreibung des Termins: ");
  ergTermin.beschreibung := SIO.GetLine();
  ergTermin.zeit  := UhrzeitEingeben();

  RETURN ergTermin;
END TerminEingeben;


PROCEDURE TerminAusgeben(termin: Termin) =
(* Ausgeben eines einzelnen Termins *)
BEGIN
  UhrzeitAusgeben(termin.zeit);
  SIO.PutText(" ");
  SIO.PutText(termin.beschreibung);
END TerminAusgeben;


PROCEDURE UhrzeitEingeben(): Uhrzeit =
(* Eingeben der Uhrzeit mit Stunden und Minuten *)
VAR ergUhrzeit: Uhrzeit;
    restZeile : TEXT;

BEGIN
  SIO.PutText("Uhrzeit (Stunden): ");
  ergUhrzeit.stunde := SIO.GetInt();
  (* Lesen der Restzeile mit RETURN *)
  restZeile := SIO.GetLine();  

  SIO.PutText("Uhrzeit (Minuten): ");
  ergUhrzeit.minute := SIO.GetInt();
  (* Lesen der Restzeile mit RETURN *)
  restZeile := SIO.GetLine();

  RETURN ergUhrzeit;  
END UhrzeitEingeben;


PROCEDURE StundenMinutenAusgeben(wert: [0..59]) =
(* Hilfsprozedur, um Stunden und Minuten immer zweistellig auszugeben *)
BEGIN
  IF wert < 10 THEN SIO.PutText("0"); END;
  SIO.PutInt(wert);
END StundenMinutenAusgeben;


PROCEDURE UhrzeitAusgeben(zeit: Uhrzeit) =
(* Gibt die Uhrzeit im Format Stunde:Minute aus *)
BEGIN
  StundenMinutenAusgeben(zeit.stunde);
  SIO.PutText(":");
  StundenMinutenAusgeben(zeit.minute);
  SIO.PutText(" Uhr");
END UhrzeitAusgeben;


PROCEDURE TerminkalenderAusgeben(termine: Terminkalender; maxIndex: INTEGER; 
                                 aufsteigend: BOOLEAN) =
(* Gibt den bis zum Eintrag maxIndex gefuellten Terminkalender termine abhaengig
   vom angegebenen BOOLEAN-Parameter aufsteigend oder absteigend aus. *)
VAR start, ende, schritt: INTEGER;

BEGIN
  IF aufsteigend THEN
    (* Initialisiere Schleifenwerte fuer aufsteigende Ausgabe *)
    start := FIRST(termine); ende := maxIndex; schritt := 1;
  ELSE
    (* Initialisiere Schleifenwerte fuer absteigende Ausgabe *)
    start := maxIndex; ende := FIRST(termine); schritt := -1;
  END;

  (* Ausgeben der Termine *)
  FOR i := start TO ende BY schritt DO
    TerminAusgeben(termine[i]); SIO.Nl();
  END;
END TerminkalenderAusgeben;


PROCEDURE TauscheTermin(VAR termin1, termin2: Termin) =
(* Hilfsprozedur, um einen einzelnen Termin zu tauschen *)
VAR speicherTermin: Termin;

BEGIN
  speicherTermin := termin1;
  termin1 := termin2;
  termin2 := speicherTermin;
END TauscheTermin;


PROCEDURE TerminkalenderSortieren(VAR termine: Terminkalender; maxIndex: INTEGER) =
(* Sortiert den bis maxIndex gefuellten Terminkalender termine in aufsteigende Reihenfolge *)

BEGIN
  (* Suche fuer jede Position im Terminkalender, ob sich auf den noch folgenden
     Positionen ein frueherer Termin findet *)
  FOR i := FIRST(termine) TO maxIndex DO
    FOR j := i + 1 TO maxIndex DO
      IF (termine[i].zeit.stunde > termine[j].zeit.stunde)  OR
         ( (termine[i].zeit.stunde = termine[j].zeit.stunde) AND
           (termine[i].zeit.minute > termine[j].zeit.minute) )
      THEN
        (* Tausche den Termin mit dem frueheren Termin *)
        TauscheTermin(termine[i], termine[j]);
      END;
    END;
  END;
END TerminkalenderSortieren;


(* Prozeduren fuer das Schreiben und Lesen der Termindatei *)

PROCEDURE TerminLesen(datei: Rd.T): Termin =
(* Liest die Daten des Termins aus einer gegebenen Datei *)
VAR ergTermin   : Termin;
    trennzeichen: CHAR;

BEGIN
  ergTermin.zeit.stunde := SIO.GetInt(datei);
  trennzeichen := SIO.GetChar(datei);
  ergTermin.zeit.minute := SIO.GetInt(datei);
  trennzeichen := SIO.GetChar(datei);
  ergTermin.beschreibung := SIO.GetLine(datei);

  RETURN ergTermin;
END TerminLesen;


PROCEDURE TerminSchreiben(datei: Wr.T; termin: Termin) =
(* Schreibt die Daten des Termins in die gegebene Datei im Format:
   Stunde:MinuteBeschreibungRETURN *)
BEGIN
  SIO.PutInt(termin.zeit.stunde, 10, datei);
  SIO.PutChar(':', datei);  (* Als Trennzeichen *)
  SIO.PutInt(termin.zeit.minute, 10, datei);
  SIO.PutChar(' ', datei);         (* Als Trennzeichen *)
  SIO.PutLine(termin.beschreibung, datei);
END TerminSchreiben;


PROCEDURE TerminkalenderLesen(datei: Rd.T; VAR termine: Terminkalender; VAR maxIndex: [0..MAXTERMIN]) =
(* Liest alle (maximal MAXTERMIN viele) Termine eines Terminkalenders aus der gegebenen Datei.
   Der gefllte Terminkalender und der maximal belegte Index werden zurueckgegeben. *)
VAR i: INTEGER := 0;
BEGIN

  WHILE NOT SIO.EOF(datei) AND (i <= MAXTERMIN) DO
    INC(i);
    termine[i] := TerminLesen(datei);
  END;  
  
  maxIndex := i;
END TerminkalenderLesen;


PROCEDURE TerminkalenderSchreiben(datei: Wr.T; termine: Terminkalender; maxIndex: INTEGER) =
(* Schreibt den bis maxIndex gefuellten Terminkalender in die gegebene Datei *)
BEGIN
  FOR i := FIRST(termine) TO maxIndex DO
    TerminSchreiben(datei, termine[i]);
  END;
END TerminkalenderSchreiben;


VAR tKalender  : Terminkalender;   
    terminIndex: [0..MAXTERMIN] := 0;

    (* Variablen fuer die zum Schreiben bzw. zum Lesen geoeffnete Datei *)
    schreibDatei: Wr.T;
    leseDatei   : Rd.T;

    (* Notwendig fuer die Benutzerinteraktion *)
    auswahl    : CHAR;
    restzeile  : TEXT;             

BEGIN
  (* Lesen der Terminkalenderdatei *)
  leseDatei := IO.OpenRead(KALENDER);
  IF (leseDatei # NIL) THEN
    TerminkalenderLesen(leseDatei, tKalender, terminIndex);
    Rd.Close(leseDatei);
  END;

  (* Benutzerinteraktion mittels REPEAT-Schleife *)
  REPEAT
    
    (* Auswahlmenue ausgeben *)
    SIO.Nl();
    SIO.PutLine("***  Terminkalender  ***");
    SIO.PutLine("(e) Neuen Termin eingeben");
    SIO.PutLine("(+) Aufsteigend sortiert ausgeben");
    SIO.PutLine("(-) Absteigend sortiert ausgeben");
    SIO.PutLine("(q) Beendet das Programm");
    SIO.Nl();

    (* Eingabe der ausgewaehlten Operation *)
    SIO.PutText("Auswahl: "); 
    auswahl   := SIO.GetChar();
    restzeile := SIO.GetLine();
    SIO.Nl();

    (* Ausgewaehlte Operation ausfuehren *)
    CASE auswahl OF
       'e', 'E' => IF terminIndex < MAXTERMIN THEN
                     (* Solange noch Platz im Terminkalender *)
                     INC(terminIndex);
                     tKalender[terminIndex] :=TerminEingeben();

                     (* Terminkalender nach Eingabe sofort sortieren *)
                     TerminkalenderSortieren(tKalender, terminIndex);
                   ELSE
                     SIO.PutLine("Terminkalender leider voll!");
                   END;

    |  '+'      => (* Terminkalender aufsteigend ausgeben *)
                   TerminkalenderAusgeben(tKalender, terminIndex, TRUE);

    |  '-'      => (* Terminkalender absteigend ausgeben *)
                   TerminkalenderAusgeben(tKalender, terminIndex, FALSE);

    |  'q', 'Q' => (* Kalenderdaten in Datei schreiben *)
                   schreibDatei := IO.OpenWrite(KALENDER);

                   IF (schreibDatei # NIL) THEN
                     TerminkalenderSchreiben(schreibDatei, tKalender, terminIndex);
                     Wr.Close(schreibDatei);
                   ELSE
                     SIO.PutLine("Fehler: Kann Datei nicht schreiben!");
                   END;

                   SIO.PutLine("Programmende!");
    ELSE
      SIO.PutLine("Ungueltige Eingabe!");
    END;
  UNTIL (auswahl = 'q') OR (auswahl = 'Q');
END TerminDatei.
















